"""Added Partner table.

Revision ID: eaed66b04913
Revises: 6e4952687205
Create Date: 2024-03-18 15:03:32.721760

"""
import sqlalchemy as sa
from alembic import op
from orchestrator.db import UtcTimestamp
from sqlalchemy.dialects import postgresql
# revision identifiers, used by Alembic.
revision = 'eaed66b04913'
down_revision = '6e4952687205'
branch_labels = None
depends_on = None

def upgrade() -> None:
    op.create_table('partners',
sa.Column('partner_id', sa.String(), server_default=sa.text('uuid_generate_v4()'), nullable=False),
         sa.Column('name', sa.String(), nullable=False),
         sa.Column('email', sa.String(), nullable=True),
         sa.Column('as_number', sa.String(), nullable=True),
         sa.Column('as_set', sa.String(), nullable=True),
         sa.Column('route_set', sa.String(), nullable=True),
         sa.Column('black_listed_as_sets', postgresql.ARRAY(sa.String()), nullable=True),
         sa.Column('additional_routers', postgresql.ARRAY(sa.String()), nullable=True),
         sa.Column('additional_bgp_speakers', postgresql.ARRAY(sa.String()), nullable=True),
         sa.Column('partner_type', sa.Enum('NREN', 'RE_PEER', 'PUBLIC_PEER', 'PRIVATE_PEER', 'UPSTREAM', 'GEANT', name='partnertype'), nullable=False),
         sa.Column('created_at', UtcTimestamp(timezone=True), server_default=sa.text('current_timestamp'), nullable=False),
         sa.Column('updated_at', UtcTimestamp(timezone=True), server_default=sa.text('current_timestamp'), nullable=False, onupdate=sa.text('current_timestamp')),

         sa.PrimaryKeyConstraint('partner_id'),
         sa.UniqueConstraint('as_number'),
         sa.UniqueConstraint('email'),
         sa.UniqueConstraint('name'),
    )

    # Add GEANT as a partner
    conn = op.get_bind()
    conn.execute(
        sa.text("""INSERT INTO partners (partner_id, name, partner_type) VALUES ('8f0df561-ce9d-4d9c-89a8-7953d3ffc961', 'GEANT', 'GEANT' )"""))

    # Add foreign key constraint to existing subscriptions table
    op.create_foreign_key(
        'fk_subscriptions_customer_id_partners',
        'subscriptions',
        'partners',  #
        ['customer_id'],
        ['partner_id']
    )

    # Make customer_id not nullable
    op.alter_column('subscriptions', 'customer_id',
                    existing_type=sa.String(length=255),
                    nullable=False)


def downgrade() -> None:
    # Drop foreign key constraint
    op.drop_constraint('fk_subscriptions_customer_id_partners', 'subscriptions', type_='foreignkey')
    # Make customer_id nullable again
    op.alter_column('subscriptions', 'customer_id',
                    existing_type=sa.String(length=255),
                    nullable=True)

    op.drop_table('partners')